#pragma once

#include <vector>

#include "glm/glm.hpp"

typedef std::vector<double> Vector6;

class Orbit
{
public:
    Orbit();
    ~Orbit() = default; 

    double getSemiMajorAxis() const;
    double getEccentricity() const;
    double getInclination() const;
    double getArgumentOfPeriapsis() const;
    double getLongitudeOfAscendingNode() const;
    double getTrueAnomaly() const;

    void setSemiMajorAxis(double semiMajorAxis);
    void setEccentricity(double eccentricity);
    void setInclination(double inclination);
    void setArgumentOfPeriapsis(double argumentOfPeriapsis);
    void setLongitudeOfAscendingNode(double longitudeOfAscendingNode);
    void setTrueAnomaly(double trueAnomaly);

    glm::dvec3 getPosition(double gravitationalParameter) const;
    //glm::dvec3 getPosition(double gravitationalParameter, double time) const;
    //glm::dvec3 getPositionFromMeanAnomaly(double meanAnomaly, double gravitationalParameter) const;

    glm::dvec3 getVelocity(double gravitationalParameter, double time) const;
    glm::dvec3 getVelocityFromMeanAnomaly(double meanAnomaly, double gravitationalParameter) const;

    //glm::dvec3 getTangent(const double meanAnomaly) const;
    //glm::mat4 getLookAlongMatrix(const double meanAnomaly) const;

    const Vector6 getCartesianCoordinates(const double gravitationalParameter) const;
    const double getMeanAnomaly(const double gravitationalParameter, const double time) const;

    void update(double time, double gravitationalParameter);

private:
    /*
     * N.B.: Order of elements is important!                       <br>
     * keplerianElements(0) = semiMajorAxis          [m]           <br>
     * keplerianElements(1) = eccentricity           [-]           <br>
     * keplerianElements(2) = inclination            [rad]         <br>
     * keplerianElements(3) = argument of periapsis  [rad]         <br>
     * keplerianElements(4) = longitude of           [rad]
     *                         ascending node        [rad]         <br>
     * keplerianElements(5) = true anomaly           [rad]
     */
    Vector6 _keplerianElements;

    const double getEccentricAnomaly() const;
};
